#!/usr/bin/perl -w
# blackcore networking
# source@source.blackcore.net
# This application will allow you to tell you which netfilter rules are
# being used between each interval.
# ipdiff written in perl for Linux.
# -rwxr-xr-x    1 root     root         3177 Jul 18  2006 /usr/local/sbin/ipdiff

my $IPTABLESS_BINARY = '/sbin/iptables-save';

# Copyright 2006 blackcore networking
=cut
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
=cut

use strict;
#use Getopt::Long;
#Getopt::Long::Configure('no_auto_abbrev');
my %config = (
	'verbosity' => 0
);
#my $result = GetOptions(
#	'v+' => \$config{'verbosity'}
#);

my @opts = getopt("@ARGV");
my $flags = 0;
my $wait_time = 1;
foreach my $opt (@opts) {
	if ($opt eq 'loop') { $flags |= 1; }
	elsif ($opt eq 'input') { $flags |= 2; }
	elsif ($opt =~ /^wait=(\d+)$/) { $wait_time = $1; }
	elsif ($opt eq 'help') {
		print "usage: $0 -loop|-input|-help\n";
		print " -loop causes application to do infinite loop\n";
		print " -input causes application to read from STDIN each loop instead of waiting N seconds\n";
		print " -wait=N causes application to wait N seconds each loop\n";
		exit 1;
	}
}
do {
	my %before = getipt();
	if ($flags & 2) {
		print "Hit enter when ready.\n";
		chomp(my $in = <STDIN>);
	} else {
		sleep $wait_time;
	}
	my %after = getipt();
	while(my($key,$val)=each %after) {
		if (defined($before{$key})) {
#			if ($key =~ /udp/ && $key =~ /DROP/ && $key =~ /69.93.7.205/) {
#				print STDERR "$key: $after{$key} / $before{$key}\n";
#			}
			
			$after{$key}-=$before{$key};
		}
	}
	my @ss = sort { $after{$a} <=> $after{$b} } keys %after;
	my($total, $intotal, $outtotal) = (0,0,0);
	for(@ss) {
		next if ($after{$_} == 0);
		if (/-A INPUT/ || /\-i/) {
			$intotal+=$after{$_};
		} elsif (/-A OUTPUT/ || /\-o/) {
			$outtotal+=$after{$_};
		}
		print fbw($after{$_})," $_\n";
		$total+=$after{$_};
	}
	print "Current Traffic: ", fbw($total), " (Inbound: ", fbw($intotal), " Outbound: ", fbw($outtotal), ")\n";
} while ($flags & 1);
sub getipt {
	my %ret;
	verbose(1, "getipt: running");
	open(FD, '-|', "$IPTABLESS_BINARY -c") or die("Cannot open iptables: $!");
	my $table;

	while(<FD>) {
		chomp;
		if (m#^\*(.+)#) {
			$table = $1;
		} elsif (/^\[(\d+):(\d+)\] (.+)/) {
			my($pkts, $bytes, $rule) = ($1, $2, $3);
#			if ($rule =~ m#69.93.7.205# && $rule =~ /udp/ && $rule =~ /DROP/) {
#				print STDERR "set: ($bytes) for $rule\n";
#			}
# BUG: Multiple rules that are the same will cause problems in counting. This will cause the script
# to skip any duplicates after the first one is found.
# 20120228 - slink
			if (defined($ret{$rule})) {
				next;
			}
			$ret{$rule} = $bytes;
		}
	}
	close(FD);
	return %ret;
}
sub fbw {
	my $in = shift;
	my $out;
	if ($in >= 1048576) {
		$out = sprintf('%.2fMB/s', $in / 1024 / 1024);
	} elsif ($in >= 1024) {
		$out = sprintf('%.2fKB/s', $in / 1024);
	} else {
		$out = sprintf('%.2fBY/s', $in);
	}
	return $out;
}
sub getopt {
        my $text = shift;
        my @flags;
        while($text =~ /-([^ ]+) ?/gs) {
                my $flag = $1;
                $text=~s/-$flag ?//g;
                push @flags, $flag;
        }
        my @newflags;
        if (!length($text)) {
                push @newflags, "";
        } else {
                push @newflags, $text;
        }
        push @newflags, @flags;
        return @newflags;
}
sub verbose {
	my($level, $message) = @_;
	if ($level > $config{'verbosity'}) { return; }
	print STDERR "$message\n";
}
